#!/usr/bin/python
## Stress-Strain Averager v5

# Copyright (c) 2015 Justin Hughes (jhughes@cavs.msstate.edu)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import sys
import os
import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import t

def stdev(data):
    a = np.zeros(len(data))
    for i in range(0,len(data)):
        a[i] = data[i]
    return (((a-a.mean())**2.0).mean()*len(a)/(len(a)-1))**0.5

def findmax(data):
    maximum = 0
    index = 0
    for item in data:
        if str(item) != 'nan':
            if item > maximum:
                maximum = item
    return maximum

def findmin(data):
    minimum = data[0]
    for item in data:
        if str(item) != 'nan':
            if item < minimum:
                minimum = item
    return minimum

def findindex(data,findthis):
    index = 0
    for i in range(0,len(data)):
        if findthis == data[i]:
            index = i
            break
    return index

if __name__ == '__main__':
    # Read data file
    nofile = True
    while True:
        filename = str(raw_input("Input File: "))
        if(os.path.isfile(filename+".csv")):
           nofile = False
        else:
            print "File not found. Try again... \n"
        try:
            data = np.genfromtxt(filename+".csv",delimiter=',',skip_header=1)
        except WindowsError:
            print "File not found. Try again... \n"
        if nofile == False:
            break
        
    np.set_printoptions(suppress=True)
    
    if(os.path.isfile(filename+"_AVERAGE.csv")):
        os.remove(filename+"_AVERAGE.csv")

    numrows = len(data[:,0])
    numcols = len(data[0,:])

    if(numcols%2 != 0):
        sys.exit()

    if(numcols < 4):
        sys.exit()

    numsets = numcols/2

    significance_level = 0.05
    num_tails = 2
    t_factor = t.ppf(1-significance_level/num_tails,numsets-1)

    stdevs = np.zeros(numrows)
    uncerts = np.zeros(numrows)
    stderr = np.zeros(numrows)
    rollsum = np.zeros(numrows)
    y_data = np.zeros(numrows)
    max_y = np.zeros(numrows)
    min_y = np.zeros(numrows)

    x_interp = data[:,0]
    y_interp = data[:,1]
    max_y += y_interp
    min_y += y_interp
    min_ef = findmax(x_interp)
    index = findindex(x_interp,min_ef)
    y_data = np.add(y_data,y_interp)
    y_data = y_data.reshape(numrows,1)
    rollsum += y_interp
    fileheader = 'Xint, Yint'
    plt.plot(x_interp,y_interp,'k-',linewidth=2)
    
    count = 0
    setcount = 1
    while(setcount < numsets):
        x_next = data[:,count+2]
        if findmax(x_next) < min_ef:
            min_ef = findmax(x_next)
            index = findindex(x_next,min_ef)
        y_next = data[:,count+3]
        y_interp_next = np.interp(x_interp,x_next,y_next)
        
        plt.plot(x_interp,y_interp_next,linewidth=2)
        y_data = np.hstack((y_data,y_interp_next.reshape(numrows,1)))
        fileheader += ",Y"+str(setcount)
        rollsum += y_interp_next
        setcount+=1
        count+=2

    # Get maximum/minimum values of interpolated data
    for i in range(0,numrows):
        max_y[i] = findmax(y_data[i,:])
        min_y[i] = findmin(y_data[i,:])
        #print max_y[i], min_y[i]
    
    AVG = rollsum/setcount
    
    for i in range(0,numrows):
        stdevs[i] = stdev(list(y_data[i,:])) 
        stderr[i] = stdevs[i]/(numsets**0.5)
        uncerts[i] = t_factor*stderr[i]
    
    upper = AVG + uncerts
    lower = AVG - uncerts
    
    maximum = 0
    for item in upper:
        if item > maximum:
            maximum = item
    
#    num_bars = int(raw_input("Number of Error Bars: "))        
#    spacing = (index+1)/num_bars
    
#    for i in range(0,numrows):
#        if i == 0:
#            uncerts[i] = 0
#        if i%spacing != 0:
#            uncerts[i] = 0
    
    plt.plot(x_interp,AVG,'k--',linewidth=3,label="Average")
    plt.plot(x_interp,upper,'r--',linewidth=3,label="%s Confidence Interval" %(1.0-significance_level))
    plt.plot(x_interp,lower,'r--',linewidth=3)
    plt.legend(loc='best')
    fileheader += ",Minimum,Maximum,Average,StDev,StErr,Uncertainty,Upper Bound,Lower Bound,t=%s" %(t_factor)
    plt.xlabel('X',fontsize=18)
    plt.ylabel('Y',fontsize=18)
    plt.ylim(0, int(1.05*maximum))
    dataout = np.hstack((x_interp.reshape(numrows,1),y_data,min_y.reshape(numrows,1),max_y.reshape(numrows,1),AVG.reshape(numrows,1), stdevs.reshape(numrows,1),stderr.reshape(numrows,1),uncerts.reshape(numrows,1),upper.reshape(numrows,1),lower.reshape(numrows,1)))
    np.savetxt(filename+"_AVERAGE.csv",dataout,fmt='%10.6f',delimiter=',',header=fileheader)
    plt.savefig(filename+"_AVG")
    
